import json

from urllib3 import connection_from_url
from urllib import urlencode


class APIClient(object):
    BASE_URL = 'http://localhost:5000/'

    def __init__(self, rate_limit_lock=None):
        self.rate_limit_lock = rate_limit_lock
        self.connection_pool = self._make_connection_pool(self.BASE_URL)

    def _make_connection_pool(self, url):
        return connection_from_url(url)

    def _compose_url(self, path, params=None):
        return self.BASE_URL + path + '?' + urlencode(params)

    def _handle_response(self, response):
        return json.loads(response.data)

    def _request(self, method, path, params=None):
        url = self._compose_url(path, params)

        self.rate_limit_lock and self.rate_limit_lock.acquire()
        r = self.connection_pool.urlopen(method.upper(), url)

        return self._handle_response(r)

    def call(self, path, **params):
        return self._request('GET', path, params=params)


class APIClient_SharedSecret(APIClient):
    API_KEY_PARAM = 'key'

    def __init__(self, api_key, *args, **kw):
        super(APIClient_SharedSecret, self).__init__(*args, **kw)
        self.api_key = api_key

    def _compose_url(self, path, params=None):
        # TODO: fix this, as per our conversation at Oct. 4, 2011, 05:10 UTC
        p = {self.API_KEY_PARAM: self.api_key}

        if params:
            p.update(params)

        return self.BASE_URL + path + '?' + urlencode(p)



import time
from threading import Lock


class RateExceededError(Exception):
    pass


class RateLimiter(object):
    def __init__(self, max_messages=10, every_seconds=1):
        self.max_messages = max_messages
        self.every_seconds = every_seconds
        self.lock = Lock()

        self._reset_window()

    def _reset_window(self):
        self.window_num = 0
        self.window_time = time.time()

    def acquire(self, block=True, timeout=None):
        self.lock.acquire()

        now = time.time()
        if now - self.window_time > self.every_seconds:
            # New rate window
            self._reset_window()

        if self.window_num >= self.max_messages:
            # Rate exceeding
            if not block:
                self.lock.release()
                raise RateExceededError()

            wait_time = self.window_time + self.every_seconds - now
            if timeout and wait_time > timeout:
                self.lock.release()
                time.sleep(timeout)

                raise RateExceededError()

            self.lock.release()
            time.sleep(wait_time)
            self.lock.acquire()

            self._reset_window()

        self.window_num += 1

        self.lock.release()



#!/usr/bin/env python

import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

def path(p):
    return os.path.join(os.path.abspath(os.path.dirname(__file__)), p)

long_description = ''

try:
    long_description += open(path('README.rst')).read()
    long_description += '\n\n' + open(path('CHANGES.rst')).read()
except IOError:
    pass

version = '1.0.2'

requirements = [
    'urllib3',
]
tests_requirements = requirements + [
    'nose',
]

setup(name='apiclient',
      version=version,
      description="Framework for making good API client libraries using urllib3.",
      long_description=long_description,
      classifiers=[
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: Software Development :: Libraries',
      ],
      keywords='api client urllib3 keepalive threadsafe http rest',
      author='Andrey Petrov',
      author_email='andrey.petrov@shazow.net',
      url='https://github.com/shazow/apiclient',
      license='MIT',
      packages=['apiclient'],
      requires=requirements,
      tests_require=tests_requirements,
      )
